@@ -14,7 +14,7 @@
 #' demo("echo", package="httpuv")
 #' }
 #' 
-#' @seealso startServer
+#' @seealso \link{startServer}
 #'   
 #' @name httpuv-package
 #' @aliases httpuv
@@ -25,6 +25,7 @@
 #' @useDynLib httpuv
 #' @import methods
 #' @importFrom Rcpp evalCpp
+#' @importFrom utils packageVersion
 NULL
 
 # Implementation of Rook input stream
@@ -130,10 +131,11 @@ rookCall <- function(func, req, data = NULL, dataLength = -1) {
     return(list(
       status=500L,
       headers=list(
-        'Content-Type'='text/plain'
+        'Content-Type'='text/plain; charset=UTF-8'
       ),
-      body=charToRaw(
-        paste("ERROR:", attr(result, "condition")$message, collapse="\n"))
+      body=charToRaw(enc2utf8(
+        paste("ERROR:", attr(result, "condition")$message, collapse="\n")
+      ))
     ))
   } else {
     return(result)
@@ -395,7 +397,7 @@ startPipeServer <- function(name, mask, app) {
 #' 
 #' @param timeoutMs Approximate number of milliseconds to run before returning. 
 #'   If 0, then the function will continually process requests without returning
-#'   unless an error occurs.
+#'   unless an error occurs. If NA, performs a non-blocking run without waiting.
 #'
 #' @examples
 #' \dontrun{
